@@ -119,6 +119,9 @@ typedef struct _OpenLocation {
 } OpenLocation;
 
 static gboolean delayed_open_location (OpenLocation *ol) {
+#ifdef DEBUG_TIMERS
+    fprintf (stderr, "delayed_open_location\n");
+#endif
     app_open_location (ol->filename, ol->app);
     g_free (ol->filename);
     if (ol->fullscreen)
@@ -395,6 +398,7 @@ static void create_menu (Application * app) {
     GtkWidget *item_playlist;
     GtkWidget *item_playlist_generators;
     GtkWidget *item_players;
+    GtkWidget *item_downloads;
 
     main_menu = HILDON_APP_MENU (hildon_app_menu_new ());
 
@@ -407,12 +411,14 @@ static void create_menu (Application * app) {
     app->menu_item_playlist_paste_in_group = makeMenuButton ("Paste in new Group");
     app->menu_item_playlist_cut = makeMenuButton("Cut");
     item_players = makeMenuButton ("Select Player");
+    item_downloads = makeMenuButton ("Download Page");
 
     hildon_app_menu_append (main_menu, GTK_BUTTON (item_new));
     hildon_app_menu_append (main_menu, GTK_BUTTON (item_open));
     hildon_app_menu_append (main_menu, GTK_BUTTON (item_playlist_generators));
     hildon_app_menu_append (main_menu, GTK_BUTTON (item_playlist));
     hildon_app_menu_append (main_menu, GTK_BUTTON (item_players));
+    hildon_app_menu_append (main_menu, GTK_BUTTON (item_downloads));
     hildon_app_menu_append (main_menu, GTK_BUTTON (app->menu_item_playlist_copy));
     hildon_app_menu_append (main_menu, GTK_BUTTON (app->menu_item_playlist_paste));
     hildon_app_menu_append (main_menu, GTK_BUTTON (app->menu_item_playlist_paste_in_group));
@@ -437,6 +443,8 @@ static void create_menu (Application * app) {
         GTK_SIGNAL_FUNC (generatorMenuActivate), app);
     g_signal_connect_after (G_OBJECT (item_players), "clicked",
         GTK_SIGNAL_FUNC (playerMenuActivate), app);
+    g_signal_connect_after (G_OBJECT (item_downloads), "clicked",
+        GTK_SIGNAL_FUNC (showDownloadPage), app);
 
     g_signal_connect (G_OBJECT (main_menu), "map-event",
         GTK_SIGNAL_FUNC (cb_playlistMenuActivate), app);
@@ -451,6 +459,9 @@ gint dbus_req_handler( const gchar *interface, const gchar *method,
     Application *appdata = (Application *) data;
     int i;
 
+#ifdef DEBUG_TIMERS
+    fprintf (stderr, "dbus_req_handler\n");
+#endif
     if (!strcmp (method, "top_application"))
         gtk_window_present (appdata->window);
     else if (!strcmp (method, "mime_open") || !strcmp (method, "load_url")) {
@@ -562,7 +573,6 @@ int main (int argc, char* argv[]) {
 
     /* Add example label to appview */
     app->hsplit_view = GTK_PANED (gtk_hpaned_new ());
-    scroll_window = GTK_SCROLLED_WINDOW (gtk_scrolled_window_new (0L, 0l));
     scroll_window = hildon_pannable_area_new ();
     g_object_set (G_OBJECT (scroll_window),
             "mov-mode", HILDON_MOVEMENT_MODE_BOTH, 0);
