@@ -52,11 +52,17 @@ void sig_usr1 (int) {
 #endif
 
 static gboolean playSchedule (void * ctr) {
+#ifdef DEBUG_TIMERS
+    debugLog() << "playSchedule" << endl;
+#endif
     static_cast <Control *> (ctr)->play ();
     return false;
 }
 
 static gboolean playCurrentSchedule (void * ctr) {
+#ifdef DEBUG_TIMERS
+    debugLog() << "playCurrentSchedule" << endl;
+#endif
     static_cast <Control *> (ctr)->playCurrent ();
     return false;
 }
@@ -139,6 +145,22 @@ Control::~Control () {
     m_process = 0L;
 }
 
+static gboolean syncSchedule (Control *ctr)
+{
+    Recents *recents = static_cast <Recents *> (ctr->recents.ptr());
+    if (!ctr->data_dir.isEmpty ()) {
+        Playlist *playlist = static_cast<Playlist *>(ctr->playlists.ptr());
+        FileDocument *downloads = (FileDocument*)ctr->downloads_doc.ptr ();
+        if (recents)
+            recents->sync (ctr->data_dir + "/recent.xml");
+        if (downloads)
+            downloads->sync (ctr->data_dir + "/downloads.xml");
+        if (playlist)
+            playlist->sync (ctr->data_dir + "/playlist.xml");
+    }
+    return !!recents;
+}
+
 void Control::init () {
     unsigned int ver, rel, req, evb, err;
     struct stat st;
@@ -243,22 +265,19 @@ void Control::init () {
     String sf = m_config->readEntry (GCONF_KEY_SAVE_FOLDER, String());
     if (!sf.isEmpty ())
         m_app->save_folder = g_strdup ((const char *)sf);
+    g_timeout_add (5*60*1000, GSourceFunc (syncSchedule), this);
 }
 
 void Control::deinit () {
     stop ();
     setProcess ((Process *) NULL);
 
+    syncSchedule (this);
+
     Recents *rec = static_cast <Recents *> (recents.ptr());
-    if (!data_dir.isEmpty () && rec->resolved &&
-            rec->load_tree_version != rec->m_tree_version)
-        rec->writeToFile (data_dir + "/recent.xml");
     rec->dispose ();
 
     Playlist *playlist = static_cast<Playlist *>(playlists.ptr());
-    if (!data_dir.isEmpty () && playlist->resolved &&
-            playlist->load_tree_version != playlist->m_tree_version)
-        playlist->writeToFile (data_dir + "/playlist.xml");
     if (m_app->save_folder)
         m_config->writeEntry (GCONF_KEY_SAVE_FOLDER, m_app->save_folder);
     playlist->dispose ();
@@ -271,6 +290,12 @@ void Control::deinit () {
         generators.remove (s);
     }
 
+    if (downloads_doc) {
+        FileDocument *doc = (FileDocument*)downloads_doc.ptr ();
+        doc->dispose ();
+        downloads_doc = NULL;
+    }
+
     viewArea ()->deleteEvent ();
 
     delete media_server;
@@ -290,6 +315,9 @@ static bool needsScreenBlanking (Application *app) {
 }
 
 gboolean cb_blankingTimer (void *p) {
+#ifdef DEBUG_TIMERS
+    debugLog() << "cb_blankingTimer" << endl;
+#endif
     Application *app = static_cast <Application *> (p);
     if (needsScreenBlanking (app)) {
         osso_display_blanking_pause (app->osso_context);
@@ -300,10 +328,16 @@ gboolean cb_blankingTimer (void *p) {
 }
 
 gboolean cb_docTimer (void * p) {
+#ifdef DEBUG_TIMERS
+    debugLog() << "cb_docTimer" << endl;
+#endif
     return static_cast <Control *> (p)->docTimerEvent ();
 }
 
 gboolean cb_scheduledUpdateTree (void * vp) {
+#ifdef DEBUG_TIMERS
+    debugLog() << "cb_scheduledUpdateTree" << endl;
+#endif
     Control *ctr = static_cast <Control *> (vp);
     if (ctr->process ()) {
         if (!ctr->playlist_data->document)
@@ -408,7 +442,7 @@ void Control::openDocument (NodePtr doc, NodePtr cur) {
     doc->document()->notify_listener = this;
     playlist_data->document = doc;
     playlist_data->current = cur ? cur : doc;
-    cur_url = doc->caption ();
+    cur_url = doc->document ()->caption ();
     app_set_title ((const char *) cur_url, m_app);
     back_request = cur;
     g_timeout_add (0, playSchedule, static_cast <void *> (this));
@@ -451,8 +485,8 @@ void Control::playCurrent () {
         updateViewAreaSizes ();
     } else {
         Mrl *mrl = back_request
-            ? back_request->mrl ()->linkNode ()
-            : playlist_data->current->mrl ()->linkNode ();
+            ? back_request->mrl ()
+            : playlist_data->current->mrl ();
         //debugLog () << "playCurrent " << m_process->playing () << " " << mrl->mrl ()->resolved << endl;
         if (m_process->playing ()) {
             errorLog() << "playCurrent and backend still playing" << endl;
@@ -519,7 +553,7 @@ ActorAgent *Control::actorAgent () const {
 bool Control::requestPlayURL (Mrl *mrl) {
     debugLog () << "requestPlayURL " << mrl->src << (mrl->view_mode == Mrl::SingleMode) << endl;
     if (process ()->state () > Process::Ready) {
-        if (process ()->mrl () == mrl->linkNode ())
+        if (process ()->mrl () == mrl)
             return true;
         back_request = mrl; // still playing, schedule it
         if (!setProcess (mrl))
@@ -664,17 +698,12 @@ void Control::stateElementChanged (Node* n, Node::State os, Node::State ns) {
         //VIEWAREA(m_app)->setEventFiltering (m_app, false);
         app_set_playing (m_app, NOT_PLAYING);
     } else if ((ns == Node::state_deactivated || ns == Node::state_finished) &&
-            n->mrl () && process ()->mrl () == n->mrl()->linkNode ()) {
+            n->mrl () && process ()->mrl () == n->mrl()) {
         process ()->stop ();
         updateViewAreaSizes (); // move away video widget
     } else if ((ns == Node::state_deferred ||
                 (os == Node::state_deferred && ns > Node::state_deferred)) &&
             n == playlist_data->document) {
-        if ((ns == Node::state_deferred &&
-                    m_process->state() != Process::Paused) ||
-                (ns > Node::state_deferred &&
-                 m_process->state() == Process::Paused))
-            m_process->pause ();
         //VIEWAREA(m_app)->setEventFiltering (m_app, true);
         app_set_playing (m_app, ns > Node::state_deferred ? PLAYING : PAUSED);
     } else if (ns == Node::state_activated ||
@@ -739,10 +768,6 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
                 stop (); // this should not happen
             }
             app_update_screen_blanking (m_app);
-            if (p == downloader && os > Process::Ready && guard &&
-                    !((Downloader *) downloader)->saved_file.isEmpty ())
-                addRecent (((Downloader *) downloader)->saved_file,
-                        mrl->caption ());
         }
         // else process changed
         if (!playlist_data->edit_mode)
@@ -763,14 +788,7 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             g_timeout_add (0, playCurrentSchedule, static_cast <void *> (this));
     } else if (ns == Process::Buffering) {
         if (mrl->view_mode != Mrl::SingleMode) {
-            AudioVideoActor *actor = mrl->media_info
-                ? static_cast <AudioVideoActor *> (mrl->media_info->media)
-                : NULL;
-            if (actor)
-                actor->ignore_pause = true;
             mrl->defer ();
-            if (actor)
-                actor->ignore_pause = false;
         }
     } else if (ns == Process::Paused &&
             playlist_data->document->state != Node::state_deferred &&
@@ -783,19 +801,11 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             playlist_data->document->defer ();
     } else if (ns == Process::Playing) {
         mrl->audio_only |= !p->hasVideo ();
-        if (mrl->state == Element::state_deferred) {
-            AudioVideoActor *actor = mrl->media_info
-                ? static_cast <AudioVideoActor *> (mrl->media_info->media)
-                : NULL;
-            if (actor)
-                actor->ignore_pause = true;
+        if (mrl->state == Element::state_deferred)
             mrl->undefer ();
-            if (actor)
-                actor->ignore_pause = false;
-        }
         setLoading (100); // move possible buffer info dialog
         addRecent (playlist_data->document->mrl()->src,
-                playlist_data->document->caption ());
+                playlist_data->document->document ()->caption ());
         updatePlayTree (m_app);
         app_update_screen_blanking (m_app);
     }
@@ -810,7 +820,8 @@ void Control::addRecent (const String &url, const String &title) {
         Recent *rec = new Recent (recents, m_app, url);
         recents->insertBefore (rec, c);
         if (!title.isEmpty ())
-            rec->setCaption (title);
+            rec->setAttribute (StringPool::attr_title, title);
+        rec->closed ();
         c = recents->firstChild ()->nextSibling ();
         int count = 1;
         Node *more = NULL;
@@ -879,8 +890,7 @@ void Control::setLoading (int perc) {
     } else {
         if (!progress_banner)
             progress_banner = hildon_banner_show_progress (
-                    GTK_WIDGET (m_app->window), NULL,
-                    m_process == downloader ? "Saving" : "Buffering");
+                    GTK_WIDGET (m_app->window), NULL, "Buffering");
         hildon_banner_set_fraction (
                 HILDON_BANNER (progress_banner), 1.0 * perc / 100);
     }
@@ -984,7 +994,7 @@ void KMPlayer::jump (Node *n, Control *ctr) {
         if (n->document() == ctr->playlist_data->document && !mrl->isPlayable())
             return;
         if (ctr->process ()->playing () &&
-                ctr->process ()->mrl() == mrl->linkNode ()) {
+                ctr->process ()->mrl() == mrl) {
             if (ctr->playlist_data->document->state == Node::state_deferred)
                 ctr->playlist_data->document->undefer (); // unpause
             else
@@ -1035,7 +1045,11 @@ void cb_playlist_prev (GtkButton * button, void *p) {
 
 void cb_playlist_play (GtkButton * button, void *p) {
     Control *ctr = (Control *)p;
-    if (ctr->current_selected)
+    if (button == GTK_BUTTON (ctr->m_app->tb_pause) ||
+            (ctr->playlist_data->document &&
+             ctr->playlist_data->document->state == Node::state_deferred))
+        ctr->play ();
+    else if (ctr->current_selected)
         jump (ctr->current_selected, ctr);
     else
         g_timeout_add (0, playSchedule, static_cast <void *> (ctr));
@@ -1092,9 +1106,11 @@ void cb_favorites_add (Application *app) {
                         id_node_html_object == mrl->id) {
                     fav->document->insertBefore (mrl->outerXML (false), NULL);
                 } else {
-                    NodePtr n = new PlaylistItem (doc, app, false, mrl->src);
-                    n->setCaption (mrl->title);
-                    fav->document->appendChild (n);
+                    PlaylistItem *i = new PlaylistItem (doc, app, false, mrl->src);
+                    if (!mrl->title.isEmpty ())
+                        i->setAttribute (StringPool::attr_title, mrl->title);
+                    i->closed ();
+                    fav->document->appendChild (i);
                 }
                 updateTree (app, fav->id);
             }
@@ -1120,20 +1136,25 @@ void cb_openUrls (void *app, int argc, gchar **argv) {
 static String makeM3u (Node *n, const String &group, const String &fn) {
     String str;
     if (id_node_playlist_item == n->id || id_node_recent_node == n->id) {
-        if (!n->mrl ()->src.isEmpty () && fn != n->mrl ()->src) {
-            if (!n->caption ().isEmpty ())
+        Mrl *mrl = n->mrl ();
+        if (!mrl->src.isEmpty () && fn != mrl->src) {
+            if (!mrl->caption ().isEmpty ())
                 str = String ("#EXTINF:,") +
                     (id_node_playlist_item == n->id && !group.isEmpty ()
                      ? group + " - "
                      : String ()) +
-                    n->caption () + String ("\r\n");
-            str += n->mrl ()->src + String ("\r\n");
+                    mrl->caption () + String ("\r\n");
+            str += mrl->src + String ("\r\n");
         }
     } else {
+        String title;
+        if (id_node_group_node == n->id) {
+            PlaylistRole *title_role = (PlaylistRole *) n->role (RolePlaylist);
+            if (title_role)
+                title = title_role->caption ();
+        }
         for (Node *c = n->firstChild (); c; c = c->nextSibling ())
-            str += makeM3u (c,
-                    id_node_group_node == n->id ? n->caption() : String (),
-                    fn);
+            str += makeM3u (c, title, fn);
     }
     return str;
 }
@@ -1475,7 +1496,6 @@ void generatorMenuActivate (GtkMenuItem *item, Application *app) {
 static void
 playlistResponse (GtkDialog *dialog, gint response, gpointer data) {
     if (response == GTK_RESPONSE_OK) {
-        Control *ctrl = (Control *) data;
         HildonTouchSelector *sel = hildon_picker_dialog_get_selector (
                 HILDON_PICKER_DIALOG (dialog));
         *(int *) data = hildon_touch_selector_get_active (
@@ -1485,7 +1505,7 @@ playlistResponse (GtkDialog *dialog, gint response, gpointer data) {
     }
 }
 
-void playlistMenuActivate (GtkMenuItem *item, Application *app) {
+void playlistMenuActivate (GtkMenuItem *, Application *app) {
     Control *ctrl = CONTROL(app);
     if (ctrl->current_selected) {
         GtkWidget *dialog = hildon_picker_dialog_new (app->window);
@@ -1533,6 +1553,245 @@ void playlistMenuActivate (GtkMenuItem *item, Application *app) {
     }
 }
 
+#include <hildon/hildon-pannable-area.h>
+#include <hildon/hildon-program.h>
+#include <hildon/hildon-window.h>
+#include <hildon/hildon-defines.h>
+#include <hildon/hildon-gtk.h>
+
+namespace {
+
+class DownloadDocument : public FileDocument
+{
+public:
+    DownloadDocument (Control *ctrl)
+        : FileDocument (id_node_download_document, "", ctrl)
+    {
+        readFromFile (ctrl->data_dir + "/downloads.xml");
+    }
+
+    virtual Node *childFromTag (const String &tag)
+    {
+        if (tag == "download")
+            return new Download (m_doc, m_control);
+        if (tag == "downloads")
+            return this;
+        return FileDocument::childFromTag (tag);
+    }
+    virtual const char *nodeName () const {
+        return "downloads";
+    }
+};
+
+}
+
+Node *Control::downloads ()
+{
+    if (!downloads_doc)
+        downloads_doc = new DownloadDocument (this);
+    return downloads_doc.ptr ();
+}
+
+void Control::updateDownloads ()
+{
+    if (m_app->downloads_view) {
+        GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (m_app->downloads_view)));
+        gtk_list_store_clear (store);
+        for (Node *c = downloads ()->firstChild (); c; c = c->nextSibling ())
+            if (id_node_download == c->id) {
+                GtkTreeIter iter;
+                gtk_list_store_append (store, &iter);
+                Download *download = static_cast <Download *> (c);
+                gtk_list_store_set (store, &iter,
+                        0, (const char *) download->title,
+                        1, download->size > 0
+                           ? 100 * download->bytes / download->size
+                           : 0,
+                        -1);
+            }
+    }
+}
+
+void Control::updateDownload (Node *node, int percentage)
+{
+    GtkTreeIter iter;
+    GtkTreeModel *model;
+    GtkListStore *store;
+
+    if (!m_app->downloads_view && !percentage)
+        showDownloadPage (NULL, m_app);
+
+    if (m_app->downloads_view) {
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (m_app->downloads_view));
+        gtk_tree_model_get_iter_first (model, &iter);
+        store = GTK_LIST_STORE (model);
+    }
+    for (Node *c = downloads ()->firstChild (); c; c = c->nextSibling ()) {
+        if (c == node) {
+            Download *download = static_cast <Download *> (c);
+            if (m_app->downloads_view)
+                gtk_list_store_set (store, &iter, 1, percentage, -1);
+            if (100 == percentage)
+                addRecent (download->file, download->title);
+            break;
+        }
+        if (id_node_download == c->id && m_app->downloads_view)
+            gtk_tree_model_iter_next (model, &iter);
+    }
+}
+
+static gboolean downloadPageDestroyed (GtkObject *, Application *app)
+{
+    debugLog() << "Download page destroyed\n";
+    app->window_downloads = NULL;
+    app->downloads_view = NULL;
+}
+
+namespace {
+
+struct DownloadInfo {
+    HildonTouchSelector *selector;
+    int selected;
+};
+
+}
+
+static void downloadResponse (GtkDialog *, gint response, gpointer data)
+{
+    DownloadInfo *info = (DownloadInfo *) data;
+    if (response == GTK_RESPONSE_ACCEPT)
+        info->selected = hildon_touch_selector_get_active (info->selector, 0);
+    else
+        info->selected = -1;
+}
+
+static void
+downloadTapped (GtkTreeView *view, GtkTreePath *path, Application *app)
+{
+    Control *ctr = CONTROL(app);
+    Download *download = NULL;
+    NodePtr safe;
+    int i = gtk_tree_path_get_indices (path) [gtk_tree_path_get_depth (path)-1];
+    for (Node *c = ctr->downloads ()->firstChild (); c; c = c->nextSibling ())
+        if (id_node_download == c->id && !i--) {
+            download = static_cast <Download *> (c);
+            safe = download;
+            break;
+        }
+    if (download) {
+        DownloadInfo info;
+        GtkWidget *dialog = gtk_dialog_new_with_buttons (download->title,
+                app->window_downloads,
+                GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
+                GTK_STOCK_OK,
+                GTK_RESPONSE_ACCEPT,
+                NULL);
+        g_signal_connect (G_OBJECT (dialog), "response",
+                G_CALLBACK (downloadResponse), &info);
+        GtkWidget *content = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+        gtk_box_set_homogeneous (GTK_BOX (content), false);
+
+        GtkWidget *widget = gtk_label_new ("");
+        gtk_label_set_justify (GTK_LABEL (widget), GTK_JUSTIFY_LEFT);
+        gtk_label_set_markup (GTK_LABEL (widget),
+                String ("Link <small><tt>") + download->uri +
+                String ("</tt></small> saved as <small><tt>") +
+                String (strrchr (download->file, '/') + 1) +
+                String ("</tt></small>"));
+        gtk_label_set_line_wrap (GTK_LABEL (widget), true);
+        gtk_label_set_selectable (GTK_LABEL (widget), true);
+        gtk_box_pack_start (GTK_BOX (content), widget, false, true, 5);
+
+        widget = hildon_touch_selector_new_text ();
+        info.selector = HILDON_TOUCH_SELECTOR (widget);
+        hildon_touch_selector_append_text (info.selector,
+                download->job
+                    ? "Suspend Download"
+                    : download->getAttribute ("size").isEmpty ()
+                        ? "Restart Download"
+                        : "Resume Download");
+        hildon_touch_selector_append_text (info.selector,
+                "Delete but keep file on disk");
+        hildon_touch_selector_append_text (info.selector,
+                "Delete and delete file on disk");
+        gtk_box_pack_start (GTK_BOX (content), widget, true, true, 10);
+
+        gtk_widget_show_all (content);
+        gtk_window_set_default_size (GTK_WINDOW (dialog), 800, 320);
+        gtk_dialog_run (GTK_DIALOG (dialog));
+        gtk_widget_destroy (dialog);
+
+        NodePtrW guard = safe;
+        safe = NULL;
+        if (guard) {
+            struct stat st;
+            switch (info.selected) {
+            case 0:
+                if (download->job) {
+                    download->job->setError ();
+                    download->job->kill (false);
+                } else {
+                    download->begin ();
+                }
+                break;
+            case 1:
+                ctr->addRecent (download->file, download->title);
+                download->parentNode ()->removeChild (download);
+                break;
+            case 2:
+                if (!stat (download->file, &st) && S_ISREG (st.st_mode))
+                    unlink (download->file);
+                download->parentNode ()->removeChild (download);
+                break;
+            }
+        }
+        ctr->updateDownloads ();
+    }
+}
+
+void showDownloadPage (GtkMenuItem *, Application *app)
+{
+    Control *ctrl = CONTROL(app);
+    if (!app->window_downloads) {
+        HildonWindow *win = HILDON_WINDOW (hildon_stackable_window_new ());
+        gtk_window_set_title (GTK_WINDOW (win), "KMPlayer Download Page");
+        g_signal_connect (G_OBJECT (win), "destroy",
+                GTK_SIGNAL_FUNC (downloadPageDestroyed), app);
+
+        GtkListStore *store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
+        app->downloads_view = hildon_gtk_tree_view_new_with_model (HILDON_UI_MODE_EDIT, GTK_TREE_MODEL (store));
+        GtkWidget *scroller = hildon_pannable_area_new ();
+
+        g_object_unref (store);
+
+        GtkTreeViewColumn *col = gtk_tree_view_column_new();
+        gtk_tree_view_column_set_alignment (col, 0.5);
+        gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
+        gtk_tree_view_column_set_fixed_width (col, 800);
+        GtkCellRenderer *render = gtk_cell_renderer_progress_new ();
+        gtk_cell_renderer_set_fixed_size (render, 800, -1);
+        gtk_tree_view_column_pack_start (col, render, false);
+        gtk_tree_view_append_column(GTK_TREE_VIEW(app->downloads_view), col);
+        gtk_tree_view_column_set_attributes(col, render,
+                "text", 0,
+                "value", 1,
+                NULL);
+        g_signal_connect (G_OBJECT (app->downloads_view),
+                "hildon-row-tapped", G_CALLBACK (downloadTapped), app);
+
+        gtk_container_add (GTK_CONTAINER (scroller), app->downloads_view);
+        gtk_container_add (GTK_CONTAINER (win), scroller);
+        gtk_widget_show_all (scroller);
+        app->window_downloads = GTK_WINDOW (win);
+        hildon_program_add_window (HILDON_PROGRAM (app->program), win);
+    }
+    ctrl->updateDownloads ();
+    gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (
+                GTK_TREE_VIEW(app->downloads_view)));
+    gtk_widget_show_all (GTK_WIDGET (app->window_downloads));
+    app_update_screen_blanking (app);
+}
+
 /*
  * TODO
 static void cb_iap (struct iap_event_t * event, void * arg) {
