@@ -101,6 +101,9 @@ public:
     void setPosition (int pos) ;
     void setLength (int len);
     void setAspect (float aspect);
+    Node *downloads ();
+    void updateDownloads ();
+    void updateDownload (Node *download, int percentage);
 
     Process *media_server;
     Process *mplayer;
@@ -120,6 +123,7 @@ public:
     NodePtrW current_selected;
     NodePtrW current_generator;
     NodeStoreList generators;
+    NodePtr downloads_doc;
     String cur_url;
     String data_dir;
     String selected_player;
