@@ -203,17 +203,6 @@ void Node::setState (State nstate) {
     }
 }
 
-bool Node::expose () const {
-    return true;
-}
-
-String Node::caption () const {
-    return String ();
-}
-
-void Node::setCaption (const String &) {
-}
-
 String Node::description () const {
     return String ();
 }
@@ -244,7 +233,7 @@ void Node::defer () {
 void Node::undefer () {
     if (state == state_deferred) {
         if (firstChild () && firstChild ()->state > state_init) {
-            state = state_began;
+            setState (state_began);
         } else {
             setState (state_activated);
             activate ();
@@ -672,13 +661,12 @@ void Attribute::setValue (const String & v) {
 
 //-----------------------------------------------------------------------------
 
-String Title::caption () const {
+String PlaylistRole::caption () const {
     return title;
 }
 
-void Title::setCaption (const String &t) {
-    setAttribute (StringPool::attr_title, t);
-    title = t;
+void PlaylistRole::setCaption (const String &s) {
+    title = s;
 }
 
 //-----------------------------------------------------------------------------
@@ -691,7 +679,7 @@ static bool hasMrlChildren (const NodePtr & e) {
 }
 
 Mrl::Mrl (NodePtr & d, short id)
-    : Title (d, id), cached_ismrl_version (~0),
+    : Element (d, id), cached_ismrl_version (~0),
       media_info (NULL),
       aspect (0), repeat (0),
       clip_begin (0),
@@ -738,10 +726,6 @@ Node *Mrl::childFromTag (const String & tag) {
     return NULL;
 }
 
-Mrl * Mrl::linkNode () {
-    return this;
-}
-
 Mrl * Mrl::mrl () {
     return this;
 }
@@ -749,7 +733,7 @@ Mrl * Mrl::mrl () {
 void Mrl::message (MessageType msg, void *content) {
     switch (msg) {
     case MsgMediaReady:
-        linkNode ()->resolved = true;
+        resolved = true;
         if (state == state_deferred) {
             if (isPlayable ()) {
                 setState (state_activated);
@@ -784,6 +768,11 @@ void *Mrl::role (RoleType msg, void *content) {
                 return p->role (msg, content);
         return NULL;
 
+    case RolePlaylist:
+        if (title.isEmpty ())
+            title = src;
+        return !title.isEmpty () ? (PlaylistRole *) this : NULL;
+
     default:
         break;
     }
@@ -791,8 +780,7 @@ void *Mrl::role (RoleType msg, void *content) {
 }
 
 void Mrl::activate () {
-    resolved |= linkNode ()->resolved;
-    if (!resolved && linkNode () == this && isPlayable ()) {
+    if (!resolved && isPlayable ()) {
         setState (state_deferred);
         media_info = new MediaInfo (this, ActorAgent::AudioVideo);
         resolved = media_info->wget (absolutePath ());
@@ -805,11 +793,7 @@ void Mrl::activate () {
 }
 
 void Mrl::begin () {
-    if (linkNode () != this) {
-        linkNode ()->activate ();
-        if (linkNode ()->unfinished ())
-            setState (state_began);
-    } else if (!src.isEmpty ()) {
+    if (!src.isEmpty ()) {
         if (!media_info)
             media_info = new MediaInfo (this, ActorAgent::AudioVideo);
         if (!media_info->media)
@@ -1326,10 +1310,6 @@ String TextNode::nodeValue () const {
     return text;
 }
 
-KDE_NO_EXPORT bool TextNode::expose () const {
-    return false;
-}
-
 //-----------------------------------------------------------------------------
 
 KDE_NO_CDTOR_EXPORT
@@ -1345,10 +1325,6 @@ Node *DarkNode::childFromTag (const String & tag) {
     return new DarkNode (m_doc, tag);
 }
 
-KDE_NO_EXPORT bool DarkNode::expose () const {
-    return false;
-}
-
 //-----------------------------------------------------------------------------
 
 GenericURL::GenericURL (NodePtr & d, const String & s, const String & name)
@@ -1390,9 +1366,13 @@ void GenericMrl::closed () {
     Mrl::closed ();
 }
 
-bool GenericMrl::expose () const {
-    return !title.isEmpty () || //return false if no caption and only one
-        previousSibling () || nextSibling ();
+void *GenericMrl::role (RoleType msg, void *content)
+{
+    if (RolePlaylist == msg)
+        return !title.isEmpty () || //return false if no title and only one
+            previousSibling () || nextSibling ()
+            ? (PlaylistRole *) this : NULL;
+    return Mrl::role (msg, content);
 }
 
 //-----------------------------------------------------------------------------
