@@ -217,13 +217,18 @@ void Process::setUserStop () {
 //-----------------------------------------------------------------------------
 
 static void watchProcessFunc (GPid pid, gint status, gpointer data) {
-    //debugLog () << " watchProcessFunc " << endl;
+#ifdef DEBUG_PROCESS_WATCH
+    debugLog () << " watchProcessFunc " << endl;
+#endif
     g_spawn_close_pid (pid);
     static_cast <UnixProcess *> (data)->processExited (pid);
 }
 
 static gboolean watchMPlayerStdin (GIOChannel *, GIOCondition, gpointer data) {
     static_cast <UnixProcess *> (data)->writeEvent ();
+#ifdef DEBUG_PROCESS_WATCH
+    debugLog () << " watchProcessStdin " << endl;
+#endif
     return true;
 }
 
@@ -232,6 +237,9 @@ static void setupProcessFunc (gpointer data) {
 
 static gboolean
 watchProcessStdout (GIOChannel *src, GIOCondition, gpointer data) {
+#ifdef DEBUG_PROCESS_WATCH
+    debugLog () << " watchProcessStdout " << endl;
+#endif
     if (G_IO_IN)
         static_cast <UnixProcess *> (data)->readEvent (src);
     return true;
@@ -500,8 +508,8 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
 }
 
 KMPLAYER_NO_EXPORT bool MPlayer::pause () {
-    wait_for_pause = !wait_for_pause;
-    debugLog() << "MPlayer::pause schedule " << wait_for_pause << endl;
+    if (state () != Paused)
+        wait_for_pause = !wait_for_pause;
     for (StringList::iterator i = commands.begin (); i != commands.end (); ++i)
         if (i->startsWith ("pause")) {
             commands.erase (i);
@@ -1149,11 +1157,107 @@ KMPLAYER_NO_EXPORT void NpStream::redirected (IOJob *, const String & uri) {
 
 #endif //__ARMEL__
 
+Download::Download (NodePtr &d, ProcessNotify *n, int _fd)
+    : Element (d, id_node_download),
+      job (NULL), notify (n),
+      size (0), bytes (0), fd (_fd)
+{}
+
+Download::~Download ()
+{
+    if (job) {
+        job->kill ();
+        ::close (fd);
+    }
+}
+
+void Download::closed ()
+{
+    file = getAttribute ("file");
+    uri = getAttribute (StringPool::attr_src);
+    title = getAttribute (StringPool::attr_title);
+    Element::closed ();
+}
+
+void Download::begin ()
+{
+    setState (state_began);
+    bytes = 0;
+    size = 0;
+    debugLog() << "Download " << uri << " as " << file << " cap:" << title << endl;
+    job = asyncGet (this, uri);
+    if (fd > -1) {
+        job->start ();
+    } else {
+        String sz = getAttribute ("size");
+        if (sz.isEmpty ()) {
+            fd = ::open (file, O_WRONLY | O_CREAT | O_TRUNC, 0664);
+        } else {
+            size = strtoll (sz, NULL, 10);
+            fd = ::open (file, O_WRONLY | O_APPEND, 0664);
+            bytes = ::lseek (fd, 0, SEEK_END);
+        }
+        if (fd > 0)
+            job->start (bytes);
+    }
+    notify->updateDownload (this, 0);
+}
+
+void Download::jobData (IOJob *jb, ByteArray &data)
+{
+    int old_perc = size > 0 ? int (100.0 * bytes / size) : 0;
+    if (!bytes) {
+        size = jb->contentLength ();
+        if (size > 0)
+            setAttribute ("size", String::number (size));
+    }
+    bytes += data.size ();
+    int new_perc = size > 0 ? int (100.0 * bytes / size) : 0;
+    if (new_perc >= 100)
+        new_perc = 99;
+    if (old_perc != new_perc)
+        notify->updateDownload (this, new_perc);
+    if (data.size ()) {
+        int nr = write (fd, data.data (), data.size ());
+        if (nr < data.size ()) {
+            String err ("Couldn't write data");
+            if (nr < 0)
+                err += Char (' ') + String (strerror (errno));
+            jb->kill (false);
+            notify->errorMsg (err);
+        }
+    }
+}
+
+void Download::jobResult (IOJob *jb)
+{
+    if (fd > 0)
+        close (fd);
+    fd = -1;
+    job = NULL;
+    if (!jb->error ()) {
+        notify->updateDownload (this, 100);
+        NodePtr safe (this);
+        parentNode ()->removeChild (this); // deletes this
+        notify->updateDownloads ();
+    }
+}
+
+void Download::redirected (IOJob * job, const String & uri)
+{
+}
+
+/*
+String saved_file;
+IOJob *job;
+ProcessNotify *notify;
+unsigned long size;
+unsigned long bytes;
+int fd;*/
+
 Downloader::Downloader (ProcessNotify *ctrl, Application *app)
  : Process (ctrl, "downloader", false),
-   job (NULL),
-   application (app),
-   fd (-1) {
+   application (app) {
 }
 
 Downloader::~Downloader () {
@@ -1187,17 +1291,21 @@ static String genFileName (const String &str, bool skip_on_invalid) {
 
 bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
     Process::play (mrl);
-    size = bytes = 0;
-    saved_file.clear ();
     if (!m_url.isEmpty ()) {
-        String name;
+        String name, title;
         int pos = m_url.lastIndexOf (Char ('/'));
         if (pos > 0)
             name = genFileName ((const char *) m_url.mid (pos + 1), false);
-        if (name.isEmpty ()) {
-            name = mrl->caption ();
-            if (!name.isEmpty ())
-                name = genFileName ((const char *) name, true);
+        PlaylistRole *title_role = (PlaylistRole*) mrl->role (RolePlaylist);
+        if (title_role) {
+            title = title_role->caption ();
+            if (name.isEmpty ())
+                name = title;
+        }
+        if (!name.isEmpty ()) {
+            if (title.isEmpty ())
+                title = name;
+            name = genFileName ((const char *) name, true);
         }
         String path = String (application->save_folder);
         if (!name.isEmpty ())
@@ -1205,12 +1313,17 @@ bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
         gchar *filename = app_file_chooser (application, (const char *) path,
                 GTK_FILE_CHOOSER_ACTION_SAVE);
         if (filename) {
-            fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
+            int fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
             if (fd > 0) {
-                job = asyncGet (this, m_url);
-                setState (Buffering);
-                saved_file = filename;
-                job->start ();
+                NodePtr doc = m_notify->downloads ();
+                Element *n = new Download (doc, m_notify, fd);
+                doc->appendChild (n);
+                n->setAttribute (StringPool::attr_src, m_url);
+                n->setAttribute (StringPool::attr_title, title);
+                n->setAttribute ("file", filename);
+                n->closed ();
+                n->begin ();
+                processNotify ()->updateDownloads ();
             } else {
                 String err ("Couldn't open file ");
                 err += String (filename) + Char (' ');
@@ -1218,8 +1331,6 @@ bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
                 processNotify ()->errorMsg (err);
             }
             app_store_save_directory (application, filename);
-            if (!saved_file.isEmpty ())
-                return true;
         }
     }
     setState (NotRunning);
@@ -1227,46 +1338,5 @@ bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
 }
 
 KMPLAYER_NO_EXPORT bool Downloader::stop () {
-    if (job)
-        job->kill ();
-    if (fd > 0)
-        close (fd);
-    job = NULL;
-    fd = -1;
-    setState (NotRunning);
     return true;
 }
-
-KMPLAYER_NO_EXPORT void Downloader::jobData (IOJob *jb, ByteArray & data) {
-    if (!bytes)
-        size = jb->contentLength ();
-    bytes += data.size ();
-    if (size > 0)
-        m_notify->setLoading ((int) (100.0 * bytes / size));
-    else if (data.size () == bytes)
-         m_notify->setLoading (0);
-    if (data.size ()) {
-        int nr = write (fd, data.data (), data.size ());
-        if (nr < data.size ()) {
-            String err ("Couldn't write data");
-            if (nr < 0)
-                err += Char (' ') + String (strerror (errno));
-            jb->kill (false);
-            processNotify ()->errorMsg (err);
-        }
-    }
-}
-
-KMPLAYER_NO_EXPORT void Downloader::jobResult (IOJob *) {
-    m_notify->setLoading (100);
-    if (fd > 0)
-        close (fd);
-    job = NULL;
-    fd = -1;
-    if (bytes == 0)
-        saved_file.clear ();
-    setState (NotRunning);
-}
-
-KMPLAYER_NO_EXPORT void Downloader::redirected (IOJob *, const String &) {
-}
