@@ -124,6 +124,9 @@ public:
     virtual void setPosition (int pos) = 0;
     virtual void setLength (int len) = 0;
     virtual void setAspect (float aspect) = 0;
+    virtual Node *downloads () = 0;
+    virtual void updateDownloads () = 0;
+    virtual void updateDownload (Node *download, int percentage) = 0;
 };
 
 class UnixProcessListener {
@@ -275,26 +278,42 @@ private:
 /*
  * Download backend
  */
-class Downloader : public Process, public IOJobListener {
+
+class Download : public Element, public IOJobListener
+{
 public:
-    Downloader (ProcessNotify * ctrl, Application *app);
+    Download (NodePtr &doc, ProcessNotify *n, int fd = -1);
+    ~Download ();
+
+    virtual void closed();
+    virtual void begin();
+    virtual const char *nodeName () const { return "download"; }
+
+    virtual void jobData (IOJob *job, ByteArray & data);
+    virtual void jobResult (IOJob *job);
+    virtual void redirected (IOJob * job, const String & uri);
+
+    String title;
+    String file;
+    String uri;
+    IOJob *job;
+    ProcessNotify *notify;
+    off_t size;
+    off_t bytes;
+    int fd;
+};
+
+class Downloader : public Process
+{
+public:
+    Downloader (ProcessNotify *ctrl, Application *app);
     ~Downloader ();
 
     virtual bool play (NodePtr mrl);
     virtual bool stop ();
 
-    String saved_file;
-
 private:
-    virtual void jobData (IOJob *job, ByteArray & data);
-    virtual void jobResult (IOJob *job);
-    virtual void redirected (IOJob * job, const String & uri);
-
-    IOJob *job;
     Application *application;
-    unsigned long size;
-    unsigned long bytes;
-    int fd;
 };
 
 } // namespace
