@@ -306,6 +306,7 @@ public:
     static GLibString fill (Char ch, int length);
     static GLibString number (int nr);
     static GLibString number (double nr);
+    static GLibString number (off_t nr);
     bool startsWith (const GLibString & prefix) const;
     bool endsWith (const GLibString & suffix) const;
     int indexOf (const GLibString &needle, int offset=-1) const;
@@ -430,9 +431,10 @@ public:
 
     void setHeader (const String &header);
     void setHttpPostData (const ByteArray &data);
-    void start ();
+    void start (off_t pos=0);
     void kill (bool quiet = true);
     bool error ();
+    void setError ();
     unsigned long contentLength() const;
     String contentType () const;
 
@@ -1027,6 +1029,12 @@ KMPLAYER_NO_EXPORT inline GLibString GLibString::number (double nr) {
     return str;
 }
 
+KMPLAYER_NO_EXPORT inline GLibString GLibString::number (off_t nr) {
+    GLibString str;
+    str.takeUtf8 (g_strdup_printf ("%lu", nr));
+    return str;
+}
+
 KMPLAYER_NO_EXPORT inline bool GLibString::isEmpty () const {
     return !data || !data->ptr [0];
 }
